<?php

class serendipity_event_spamblock_bayes extends serendipity_event {

    var $b8 = null;

    function introspect(&$propbag) {
        global $serendipity;

        $this->title = 'Spamblock Bayes';

        $propbag->add('name',          $this->title);
        $propbag->add('version',       '0.1');
        $propbag->add('event_hooks',    array('frontend_saveComment' => true,
                                              'backend_approvecomment' => true));
        $propbag->add('groups', array('ANTISPAM'));
    }

    function generate_content(&$title) {
        $title = $this->title;
    }

    function install() {
        global $serendipity;
        if(! file_exists(dirname(__FILE__) . '/b8/wordlist.db')) {
            $this->setCreate('TRUE');
        }
        
        $this->startb8();
        //did something to create the table
        $this->setCreate('FALSE');
        $this->learnFromOld();
        
    }

    function startb8() {
        @include_once dirname(__FILE__) . '/b8/b8.php';
        if(!$this->b8->constructed) {
            $this->b8 = new b8;
            if(!$this->b8->constructed) {
                echo "<strong>error:</strong> Could not initialize b8. Truncating.";
                return false;
            }
        }
        return true;
    }

    function learnFromOld() {
        //approved comments are ham
        $sql = "SELECT
                    author,email,url,body
                FROM
                    serendipity_comments
                WHERE
                    status = 'approved';";
        $ham_comments = serendipity_db_query($sql);
        //learn via the spamblock-log what is spam:
        $sql = "SELECT
                    author,email,url,body
                FROM
                    serendipity_spamblocklog
                WHERE
                    type = 'REJECTED';";
        $spam_comments = serendipity_db_query($sql);
        //$this->debugMsg($ham_comments);
        foreach($ham_comments as $ham) {
            foreach($ham as $element) {
                $this->b8->learn($element, "ham");
            }
        }
        //$this->debugMsg($spam_comments);
        foreach($spam_comments as $spam) {
            foreach($spam as $element) {
                $this->b8->learn($element, "spam");
            }
        }
    }

    /* Set createDB to FALSE or TRUE
     * param $value FALSE or TRUE, value to set
     * */
    function setCreate($value) {
        $f = dirname(__FILE__) . '/b8/etc/config_storage';
        $config = file_get_contents($f);
        if ($value == 'TRUE') {
            $old = 'FALSE';
        } else {
            $old = 'TRUE';
        }
        $config = str_replace('createDB = '. $old, 'createDB = '.$value, $config);
        $fp = fopen($f, 'w');
        flock($fp, LOCK_EX);
        fwrite($fp, $config);
        fclose($fp);
    }

    function isSpam(&$comment) {
        if (is_array($comment)) {
            $comment = implode($comment);
        }
        $rating = $this->b8->classify($comment);
        //a rating greater 0.8 is probably spam,
        //but maybe this barrier is too low (use 0.9?)
        if ($rating >= 0.8) {
            return true;
        }
        return false;
    }

    function event_hook($event, &$bag, &$eventData, $addData = null) {
        global $serendipity;

        $hooks = &$bag->get('event_hooks');
        if(!$this->startb8()) {
            $this->debugMsg("error starting b8 \n");
            return false;
        }
        if (isset($hooks[$event])) {
            switch($event) {
                case 'frontend_saveComment':
                    if (!is_array($eventData) || serendipity_db_bool($eventData['allow_comments'])) {
                        $serendipity['csuccess'] = 'true';
                        if ($this->isSpam($addData)) {
                            $eventData = array('allow_comments' => false);
                            $serendipity['messagestack']['comments'][] = PLUGIN_EVENT_SPAMBLOCK_ERROR_BAYES;
                            return false;
                          }
                    }

                    return true;
                    break;
                    
                case 'backend_approvecomment':
                    //use aprovement to learn what is not spam
                    if(! empty($eventData['author'])) {
                        $this->b8->learn($eventData['author'], "ham");
                    }
                    if(! empty($eventData['email'])) {
                        $this->b8->learn($eventData['email'], "ham");
                        
                    }
                    if(! empty($eventData['url'])) {
                        $this->b8->learn($eventData['url'], "ham");
                        
                    }
                    if(! empty($eventData['body'])) {
                        $this->b8->learn($eventData['body'], "ham");
                    }
                    return true;
                    break;
                default:
                    return false;
                    break;
            }
        } else {
            return false;
        }
    }

    function debugMsg($msg) {
        global $serendipity;

        $this->debug_fp = @fopen($serendipity['serendipityPath'] . 'templates_c/spamblock_bayes.log', 'a');
        if (!$this->debug_fp) {
            return false;
        }

        if (empty($msg)) {
            fwrite($this->debug_fp, "failure \n");
        } else {
            fwrite($this->debug_fp, print_r($msg, true));
        }
        fclose($this->debug_fp);
    }
}
